@@ -511,7 +511,7 @@ static gboolean avdtp_send(struct avdtp *session, uint8_t transaction,
 	cont_fragments = (len - (session->omtu - sizeof(start))) /
 					(session->omtu - sizeof(cont)) + 1;
 
-	debug("avdtp_send: %u bytes split into %d fragments", len,
+	debug("avdtp_send: %zu bytes split into %d fragments", len,
 							cont_fragments + 1);
 
 	/* Send the start packet */
@@ -529,7 +529,7 @@ static gboolean avdtp_send(struct avdtp *session, uint8_t transaction,
 	if (!try_send(sock, session->buf, session->omtu))
 		return FALSE;
 
-	debug("avdtp_send: first packet with %d bytes sent",
+	debug("avdtp_send: first packet with %lu bytes sent",
 						session->omtu - sizeof(start));
 
 	sent = session->omtu - sizeof(start);
@@ -614,7 +614,7 @@ static gboolean disconnect_timeout(gpointer user_data)
 	stream_setup = session->stream_setup;
 	session->stream_setup = FALSE;
 
-	dev = manager_get_device(&session->server->src, &session->dst);
+	dev = manager_get_device(&session->server->src, &session->dst, FALSE);
 
 	if (dev && dev->sink && stream_setup)
 		sink_setup_stream(dev->sink, session);
@@ -711,7 +711,7 @@ static void avdtp_set_state(struct avdtp *session,
 	session->state = new_state;
 
 	avdtp_get_peers(session, &src, &dst);
-	dev = manager_get_device(&src, &dst);
+	dev = manager_get_device(&src, &dst, FALSE);
 	if (dev == NULL) {
 		error("avdtp_set_state(): no matching audio device");
 		return;
@@ -856,6 +856,12 @@ static void avdtp_sep_set_state(struct avdtp *session,
 	struct avdtp_stream *stream = sep->stream;
 	avdtp_state_t old_state;
 	struct avdtp_error err, *err_ptr = NULL;
+	GSList *l;
+
+	if (!stream) {
+		error("Error changing sep state: stream not available");
+		return;
+	}
 
 	if (sep->state == state) {
 		avdtp_error_init(&err, AVDTP_ERROR_ERRNO, EIO);
@@ -871,13 +877,9 @@ static void avdtp_sep_set_state(struct avdtp *session,
 	old_state = sep->state;
 	sep->state = state;
 
-	if (stream) {
-		GSList *l;
-		for (l = stream->callbacks; l != NULL; l = g_slist_next(l)) {
-			struct stream_callback *cb = l->data;
-			cb->cb(stream, old_state, state, err_ptr,
-					cb->user_data);
-		}
+	for (l = stream->callbacks; l != NULL; l = g_slist_next(l)) {
+		struct stream_callback *cb = l->data;
+		cb->cb(stream, old_state, state, err_ptr, cb->user_data);
 	}
 
 	switch (state) {
@@ -1221,7 +1223,7 @@ static gboolean avdtp_setconf_cmd(struct avdtp *session, uint8_t transaction,
 	}
 
 	avdtp_get_peers(session, &src, &dst);
-	dev = manager_get_device(&src, &dst);
+	dev = manager_get_device(&src, &dst, FALSE);
 	if (!dev) {
 		error("Unable to get a audio device object");
 		goto failed;
@@ -1650,7 +1652,7 @@ static enum avdtp_parse_result avdtp_parse_data(struct avdtp *session,
 	switch (header->packet_type) {
 	case AVDTP_PKT_TYPE_SINGLE:
 		if (size < sizeof(*single)) {
-			error("Received too small single packet (%d bytes)", size);
+			error("Received too small single packet (%zu bytes)", size);
 			return PARSE_ERROR;
 		}
 		if (session->in.active) {
@@ -1671,7 +1673,7 @@ static enum avdtp_parse_result avdtp_parse_data(struct avdtp *session,
 		break;
 	case AVDTP_PKT_TYPE_START:
 		if (size < sizeof(*start)) {
-			error("Received too small start packet (%d bytes)", size);
+			error("Received too small start packet (%zu bytes)", size);
 			return PARSE_ERROR;
 		}
 		if (session->in.active) {
@@ -1692,7 +1694,7 @@ static enum avdtp_parse_result avdtp_parse_data(struct avdtp *session,
 		break;
 	case AVDTP_PKT_TYPE_CONTINUE:
 		if (size < sizeof(struct avdtp_continue_header)) {
-			error("Received too small continue packet (%d bytes)",
+			error("Received too small continue packet (%zu bytes)",
 									size);
 			return PARSE_ERROR;
 		}
@@ -1715,7 +1717,7 @@ static enum avdtp_parse_result avdtp_parse_data(struct avdtp *session,
 		break;
 	case AVDTP_PKT_TYPE_END:
 		if (size < sizeof(struct avdtp_continue_header)) {
-			error("Received too small end packet (%d bytes)", size);
+			error("Received too small end packet (%zu bytes)", size);
 			return PARSE_ERROR;
 		}
 		if (!session->in.active) {
@@ -1785,7 +1787,7 @@ static gboolean session_cb(GIOChannel *chan, GIOCondition cond,
 	}
 
 	if (size < sizeof(struct avdtp_common_header)) {
-		error("Received too small packet (%d bytes)", size);
+		error("Received too small packet (%zu bytes)", size);
 		goto failed;
 	}
 
@@ -2043,7 +2045,6 @@ static void auth_cb(DBusError *derr, void *user_data)
 
 static void avdtp_confirm_cb(GIOChannel *chan, gpointer data)
 {
-	int sk;
 	struct avdtp *session;
 	struct audio_device *dev;
 	char address[18];
@@ -2065,8 +2066,8 @@ static void avdtp_confirm_cb(GIOChannel *chan, gpointer data)
 	debug("AVDTP: incoming connect from %s", address);
 
 	session = avdtp_get_internal(&src, &dst);
-
-	sk = g_io_channel_unix_get_fd(chan);
+	if (!session)
+		goto drop;
 
 	if (session->pending_open && session->pending_open->open_acp) {
 		if (!bt_io_accept(chan, avdtp_connect_cb, session, NULL, NULL))
@@ -2079,7 +2080,7 @@ static void avdtp_confirm_cb(GIOChannel *chan, gpointer data)
 		goto drop;
 	}
 
-	dev = manager_get_device(&src, &dst);
+	dev = manager_get_device(&src, &dst, TRUE);
 	if (!dev) {
 		error("Unable to get audio device object for %s", address);
 		goto drop;
@@ -3210,15 +3211,11 @@ int avdtp_unregister_sep(struct avdtp_local_sep *sep)
 	if (!sep)
 		return -EINVAL;
 
-	if (sep->info.inuse)
-		return -EBUSY;
-
 	server = sep->server;
 	server->seps = g_slist_remove(server->seps, sep);
 
 	if (sep->stream)
-		avdtp_sep_set_state(sep->stream->session, sep,
-							AVDTP_STATE_IDLE);
+		release_stream(sep->stream, sep->stream->session);
 
 	g_free(sep);
 
@@ -3346,11 +3343,19 @@ int avdtp_init(const bdaddr_t *src, GKeyFile *config)
 void avdtp_exit(const bdaddr_t *src)
 {
 	struct avdtp_server *server;
+	GSList *l;
 
 	server = find_server(servers, src);
 	if (!server)
 		return;
 
+	for (l = sessions; l; l = l->next) {
+		struct avdtp *session = l->data;
+
+		if (session->server == server)
+			connection_lost(session, -ECONNABORTED);
+	}
+
 	servers = g_slist_remove(servers, server);
 
 	g_io_channel_shutdown(server->io, TRUE, NULL);
