@@ -74,6 +74,8 @@
 static DBusConnection *connection = NULL;
 static GSList *adapter_drivers = NULL;
 
+const struct btd_adapter_ops *adapter_ops = NULL;
+
 struct session_req {
 	struct btd_adapter	*adapter;
 	DBusConnection		*conn;		/* Connection reference */
@@ -403,25 +405,16 @@ static int set_mode(struct btd_adapter *adapter, uint8_t new_mode)
 
 	scan_enable = mode2scan(new_mode);
 
+	if (!adapter->up && scan_enable != SCAN_DISABLED) {
+		err = adapter_ops->start(adapter->dev_id);
+		if (err < 0)
+			return err;
+	}
+
 	dd = hci_open_dev(adapter->dev_id);
 	if (dd < 0)
 		return -EIO;
 
-	if (!adapter->up && scan_enable != SCAN_DISABLED) {
-		/* Start HCI device */
-		if (ioctl(dd, HCIDEVUP, adapter->dev_id) == 0)
-			goto done; /* on success */
-
-		if (errno != EALREADY) {
-			err = -errno;
-			error("Can't init device hci%d: %s (%d)\n",
-				adapter->dev_id, strerror(errno), errno);
-
-			hci_close_dev(dd);
-			return err;
-		}
-	}
-
 	if (adapter->up && scan_enable == SCAN_DISABLED) {
 		struct hci_request rq = {
 			.ogf = OGF_HOST_CTL,
@@ -1243,7 +1236,7 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	dict_append_entry(&dict, "Class", DBUS_TYPE_UINT32, &class);
 
 	/* Powered */
-	value = adapter->up ? TRUE : FALSE;
+	value = (adapter->up && !adapter->off_requested) ? TRUE : FALSE;
 	dict_append_entry(&dict, "Powered", DBUS_TYPE_BOOLEAN, &value);
 
 	/* Discoverable */
@@ -1603,7 +1596,12 @@ static DBusMessage *remove_device(DBusConnection *conn,
 				ERROR_INTERFACE ".DoesNotExist",
 				"Device creation in progress");
 
-	adapter_remove_device(conn, adapter, device);
+	device_set_temporary(device, TRUE);
+
+	if (device_is_connected(device))
+		device_disconnect(device);
+	else
+		adapter_remove_device(conn, adapter, device);
 
 	return dbus_message_new_method_return(msg);
 }
@@ -2042,7 +2040,8 @@ static int adapter_up(struct btd_adapter *adapter, int dd)
 			goto proceed;
 		}
 
-		if (read_on_mode(srcaddr, mode, sizeof(mode)) < 0)
+		if (read_on_mode(srcaddr, mode, sizeof(mode)) < 0 ||
+						g_str_equal(mode, "off"))
 			write_device_mode(&adapter->bdaddr, "connectable");
 		else
 			write_device_mode(&adapter->bdaddr, mode);
@@ -2057,15 +2056,10 @@ static int adapter_up(struct btd_adapter *adapter, int dd)
 	}
 
 proceed:
-	if (dev_down == FALSE) {
+	if (dev_down == FALSE)
 		hci_send_cmd(dd, OGF_HOST_CTL, OCF_WRITE_SCAN_ENABLE,
 				1, &scan_mode);
 
-		emit_property_changed(connection, adapter->path,
-					ADAPTER_INTERFACE, "Powered",
-					DBUS_TYPE_BOOLEAN, &powered);
-	}
-
 	if (adapter->initialized == FALSE) {
 		load_drivers(adapter);
 		load_devices(adapter);
@@ -2077,12 +2071,18 @@ proceed:
 		adapter->initialized = TRUE;
 
 		adapter_update(adapter, 0, FALSE);
+
+		manager_add_adapter(adapter->path);
 	}
 
 	if (dev_down) {
 		ioctl(dd, HCIDEVDOWN, adapter->dev_id);
+		adapter->off_requested = TRUE;
 		return 1;
-	}
+	} else
+		emit_property_changed(connection, adapter->path,
+					ADAPTER_INTERFACE, "Powered",
+					DBUS_TYPE_BOOLEAN, &powered);
 
 	return 0;
 }
@@ -2450,12 +2450,12 @@ void adapter_remove(struct btd_adapter *adapter)
 
 	debug("Removing adapter %s", path);
 
-	unload_drivers(adapter);
-
 	for (l = adapter->devices; l; l = l->next)
 		device_remove(l->data, connection, FALSE);
 	g_slist_free(adapter->devices);
 
+	unload_drivers(adapter);
+
 	/* Return adapter to down state if it was not up on init */
 	if (adapter->up && !adapter->already_up) {
 		int dd = hci_open_dev(adapter->dev_id);
@@ -2520,6 +2520,11 @@ int adapter_get_state(struct btd_adapter *adapter)
 	return adapter->state;
 }
 
+gboolean adapter_is_ready(struct btd_adapter *adapter)
+{
+	return adapter->initialized;
+}
+
 struct remote_dev_info *adapter_search_found_devices(struct btd_adapter *adapter,
 						struct remote_dev_info *match)
 {
@@ -3073,3 +3078,30 @@ gboolean adapter_powering_down(struct btd_adapter *adapter)
 {
 	return adapter->off_requested;
 }
+
+int btd_register_adapter_ops(struct btd_adapter_ops *btd_adapter_ops)
+{
+	/* Already registered */
+	if (adapter_ops)
+		return -EALREADY;
+
+	if (btd_adapter_ops->setup == NULL)
+		return -EINVAL;
+
+	adapter_ops = btd_adapter_ops;
+
+	return 0;
+}
+
+void btd_adapter_cleanup_ops()
+{
+	adapter_ops->cleanup();
+}
+
+int adapter_ops_setup()
+{
+	if (!adapter_ops)
+		return -EINVAL;
+
+	return adapter_ops->setup();
+}
